'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Ctrl = function () {
  function Ctrl(_ref) {
    var getModel = _ref.getModel;

    _classCallCheck(this, Ctrl);

    this.events = {};
    this._usedModels = [];
    this.state = {
      __updateStateCounter: 0
    };
    this.ee = new _eventemitter2['default']();

    this._getModel = getModel;
    this.loadEvents();
    this.updateComputedState();
  }

  Ctrl.prototype.loadEvents = function loadEvents() {
    var name = this.getName();
    for (var eventName in this.events) {
      var handlerName = this.events[eventName];
      var handler = this[handlerName].bind(this);
      this.ee.on(name + '.' + eventName, handler);
    }
  };

  Ctrl.prototype.getName = function getName() {
    return this.name || this.constructor.name;
  };

  Ctrl.prototype.getModel = function getModel(name) {
    var model = this._getModel(name);
    if (model == void 0) throw new Error('cannot find model named ' + name);
    if (this._usedModels.indexOf(name) === -1) this._usedModels.push(name);
    this.initModel(model);
    return model;
  };

  Ctrl.prototype.initModel = function initModel(model) {
    var _this = this;

    model.on('beforeUpdate', function (state, newState) {
      _this.beforeModelUpdate(name, state, newState);
    });
    model.on('didUpdate', function (state, prevState) {
      _this.didModelUpdate(name, state, prevState);
    });
  };

  Ctrl.prototype.onComputeDone = function onComputeDone(func) {
    this.ee.on('didCompute', func);
  };

  Ctrl.prototype.updateComputedState = function updateComputedState() {
    this.state = _extends({}, this.compute(), {
      __updateStateCounter: this.state.__updateStateCounter + 1
    });
    this.ee.emit('didCompute');
  };

  Ctrl.prototype.didModelUpdate = function didModelUpdate(modelName, state, prevState) {};

  Ctrl.prototype.beforeModelUpdate = function beforeModelUpdate(modelName, state, newState) {};

  Ctrl.prototype.compute = function compute() {};

  return Ctrl;
}();

exports['default'] = Ctrl;