"use strict";

exports.__esModule = true;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Store = function () {
  function Store(_ref) {
    var models = _ref.models;
    var Ctrls = _ref.Ctrls;

    _classCallCheck(this, Store);

    this.loadModels(models);
    this.loadCtrls(Ctrls);
  }

  Store.prototype.loadModels = function loadModels(models) {
    this.models = models;
  };

  Store.prototype.loadCtrls = function loadCtrls(Ctrls) {
    var getModel = this.getModel.bind(this);
    var ctrls = {};
    for (var i = 0, l = Ctrls.length; i < l; i++) {
      var Ctrl = Ctrls[i];
      var ctrl = Ctrl({ getModel: getModel });
      var ctrlName = ctrl.getName();
      if (ctrls[ctrlName]) throw new Error(ctrlName + " has been declared");
    }
    this.ctrls = ctrls;
  };

  Store.prototype.getCtrls = function getCtrls() {
    var _this = this;

    for (var _len = arguments.length, ctrlNames = Array(_len), _key = 0; _key < _len; _key++) {
      ctrlNames[_key] = arguments[_key];
    }

    return ctrlNames.map(function (ctrlName) {
      return _this.ctrls[ctrlName];
    });
  };

  Store.prototype.getModel = function getModel(name) {
    return this.models[name];
  };

  Store.prototype.invoke = function invoke(ctrlName, event) {
    var _ctrls$ctrlName;

    for (var _len2 = arguments.length, args = Array(_len2 > 2 ? _len2 - 2 : 0), _key2 = 2; _key2 < _len2; _key2++) {
      args[_key2 - 2] = arguments[_key2];
    }

    (_ctrls$ctrlName = this.ctrls[ctrlName]).emit.apply(_ctrls$ctrlName, [event].concat(args));
  };

  Store.prototype.bind = function bind(ctrlName, event) {
    for (var _len3 = arguments.length, bindArgs = Array(_len3 > 2 ? _len3 - 2 : 0), _key3 = 2; _key3 < _len3; _key3++) {
      bindArgs[_key3 - 2] = arguments[_key3];
    }

    var _this2 = this;

    return function () {
      for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
        args[_key4] = arguments[_key4];
      }

      _this2.invoke.apply(_this2, [ctrlName, event].concat([bindArgs.concat(args)]));
    };
  };

  return Store;
}();

exports["default"] = Store;