import EventEmitter from 'eventemitter3'

export default class Ctrl {

  events = {}

  _usedModels = []

  state = {
    __updateStateCounter: 0
  }

  ee = new EventEmitter()

  constructor({getModel}) {
    this._getModel = getModel
    this.loadEvents()
    this.updateComputedState()
  }

  loadEvents() {
    const name = this.getName()
    for (let eventName in this.events) {
      const handlerName = this.events[eventName]
      const handler = this[handlerName].bind(this)
      this.ee.on(`${name}.${eventName}`, handler)
    }
  }

  getName() {
    return this.name || this.constructor.name
  }

  getModel(name) {
    const model = this._getModel(name)
    if (model == void 0) throw new Error(`cannot find model named ${name}`)
    if (this._usedModels.indexOf(name) === -1) this._usedModels.push(name)
    this.initModel(model)
    return model
  }

  initModel(model) {
    model.on('beforeUpdate', (state, newState) => {
      this.beforeModelUpdate(name, state, newState)
    })
    model.on('didUpdate', (state, prevState) => {
      this.didModelUpdate(name, state, prevState)
    })
  }

  onComputeDone(func) {
    this.ee.on('didCompute', func)
  }

  updateComputedState() {
    this.state = {
      ...this.compute(),
      __updateStateCounter: this.state.__updateStateCounter + 1
    }
    this.ee.emit('didCompute')
  }

  didModelUpdate(modelName, state, prevState) {
  }

  beforeModelUpdate(modelName, state, newState) {
  }

  compute() {
  }
}