export default class Store {
  constructor({models, Ctrls}) {
    this.loadModels(models)
    this.loadCtrls(Ctrls)
  }

  loadModels(models){
    this.models = models
  }

  loadCtrls(Ctrls) {
    const getModel = this.getModel.bind(this)
    const ctrls = {}
    for (let i = 0, l = Ctrls.length; i < l; i++) {
      const Ctrl = Ctrls[i]
      const ctrl = Ctrl({getModel})
      const ctrlName = ctrl.getName()
      if (ctrls[ctrlName]) throw new Error(`${ctrlName} has been declared`)
    }
    this.ctrls = ctrls
  }

  getCtrls(...ctrlNames) {
    return ctrlNames.map((ctrlName) => {
      return this.ctrls[ctrlName]
    })
  }

  getModel(name) {
    return this.models[name]
  }

  invoke(ctrlName, event, ...args) {
    this.ctrls[ctrlName].emit(event, ...args)
  }

  bind(ctrlName, event, ...bindArgs) {
    return (...args) => {
      this.invoke(ctrlName, event, ...[bindArgs.concat(args)])
    }
  }
}